//--------------------------------------------------------------------
// TUCN, Computer Science Department
// Input/Output Systems and Peripheral Devices
//--------------------------------------------------------------------
// http://users.utcluj.ro/~baruch/en/pages/teaching/inputoutput-systems/laboratory.php
//--------------------------------------------------------------------
// File:		PortAccess-e.cpp
// Date:		30.09.2009
// Modified:	30.09.2009
//--------------------------------------------------------------------
// Acquiring access to a number of 8 consecutive I/O ports 
//--------------------------------------------------------------------
// - Example to illustrate port access by setting the I/O permission 
//   bitmap (IOPM)
// - The address of the first port in the group of 8 ports is defined by PortAddr
// - The PortTalk driver is used 
//--------------------------------------------------------------------
// - Win32 API functions used:
//   CreateFile, DeviceIoControl, CloseHandle, GetLastError
//--------------------------------------------------------------------
// - Operating system: Microsoft Windows 2000/XP
// - Compiler: Microsoft Visual Studio 2008
//--------------------------------------------------------------------

#include <stdio.h>
#include <windows.h>
#include <winioctl.h>
#include "pt_ioctl.c"

int __cdecl main (int argc, char* argv[])
{
    WORD PortAddr = 0x1F0;		// Address of first port to acquire access
	HANDLE PortTalk_Handle;		// Handle for PortTalk driver
    int Error;					// Error code for the DeviceIoControl() function
    int Offset;					// Offset in the I/O permission bitmap
    DWORD BytesReturned;		// Bytes returned for the DeviceIoControl() function

    printf ("Port access using the PortTalk driver V2.0\n");
   
    // Open PortTalk Driver

    PortTalk_Handle = CreateFile ("\\\\.\\PortTalk", 
                                 GENERIC_READ, 
                                 0, 
                                 NULL,
                                 OPEN_EXISTING, 
                                 FILE_ATTRIBUTE_NORMAL, 
                                 NULL);

    if (PortTalk_Handle == INVALID_HANDLE_VALUE) {
		printf ("Could not access the PortTalk driver; the driver is not loaded\n\n");
		return -1;
    }

	Offset = PortAddr / 8;
	Error = DeviceIoControl (PortTalk_Handle,
							 IOCTL_SET_IOPM,
							 &Offset,
							 3,
							 NULL,
							 0,
							 &BytesReturned,
							 NULL);

	if (!Error) printf ("Error %d granting access to port 0x%03X\n", GetLastError(), PortAddr);
	else        printf ("Port 0x%03X (IOPM offset 0x%02X) has been granted access\n", PortAddr, Offset);

    CloseHandle (PortTalk_Handle);

	// Read from a port with access granted

	OpenPortTalk();
	inportb (PortAddr+7);
	ClosePortTalk();

    return 0;
}
